# Generated by Django 5.1.4 on 2025-01-10 18:08

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flex_report', '0019_templatesavedfilter'),
    ]

    operations = [
        migrations.AddField(
            model_name='templatesavedfilter',
            name='created',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now, verbose_name='Created'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='templatesavedfilter',
            name='modified',
            field=models.DateTimeField(auto_now=True, verbose_name='Modified'),
        ),
    ]
