# Generated by Django 5.0 on 2024-10-19 05:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flex_report", "0017_remove_column_groups_remove_column_users_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="tablebutton",
            name="title",
            field=models.CharField(
                default="", max_length=255, unique=True, verbose_name="title"
            ),
        ),
    ]
