# Generated by Django 5.1.5 on 2025-01-19 20:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('flex_report', '0023_alter_template_created_date_and_more'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='templatesavedfilter',
            unique_together={('title', 'template')},
        ),
        migrations.RemoveField(
            model_name='templatesavedfilter',
            name='slug',
        ),
    ]
