# Generated by Django 5.1.4 on 2025-01-13 12:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flex_report', '0022_alter_templatesavedfilter_unique_together_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='template',
            name='created_date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Created Date'),
        ),
        migrations.AlterField(
            model_name='template',
            name='modified_date',
            field=models.DateTimeField(auto_now=True, verbose_name='Modified Date'),
        ),
    ]
